// You can import API functions like this from D3.js.
import { select,  
          brush,
          csv, 
          scaleLinear, 
          axisBottom, 
          axisLeft,
          scaleOrdinal,
          event,
          extent,
          selection,
          timeParse, 
          timeFormat
       } from 'd3';

// You can import local modules like this. See message.js!
import { message } from './myMessage';
// This line uses D3 to set the text of   the message div.

var margin = {top: 10, right: 30, bottom: 30, left: 60},
    width = 460 - margin.left - margin.right,
    height = 400 - margin.top - margin.bottom;

const parseDate = timeParse("%d/%m/%Y");
const formatDate = timeFormat("%b %d");

// append the svg object to the body of the page
var Svg = select("#dataviz_brushZoom")
  .append("svg")
    .attr("width", width + margin.left + margin.right)
    .attr("height", height + margin.top + margin.bottom)
  .append("g")
    .attr("transform",
          "translate(" + margin.left + "," + margin.top + ")");

//Read the data

csv("https://vizhub.com/careyshan/datasets/borehole_dummydata1.csv", function(data) {
  data.forEach(d => {
      d.Depth = +d.Depth;
      d.Date = parseDate(d.Date);
  })

  
  const xAxisTickFormat = data =>
    formatDate(data);
  
  
  // Add X axis
  var x = scaleLinear()
    .domain(extent(data, d => d.Date))
    .range([0, width])
    .nice();
 
  var xAxis = Svg.append("g")
    .attr("transform", "translate(0," + height + ")")
    .call(axisBottom(x)
    .tickFormat(xAxisTickFormat));

  // Add Y axis
  var y = scaleLinear()
    .domain([0, 9])
    .range([ height, 0]);
  Svg.append("g")
    .call(axisLeft(y));

  // Add a clipPath: everything out of this area won't be drawn.
  var clip = Svg.append("defs").append("svg:clipPath")
      .attr("id", "clip")
      .append("svg:rect")
      .attr("width", width )
      .attr("height", height )
      .attr("x", 0)
      .attr("y", 0);

  // Color scale: give me a specie name, I return a color
  var color = scaleOrdinal()
    .domain(["setosa", "versicolor", "virginica" ])
    .range([ "#440154ff", "#21908dff", "#fde725ff"])

  // Add brushing
  var brush = d3.brushX()                 // Add the brush feature using the d3.brush function
      .extent( [ [0,0], [width,height] ] ) // initialise the brush area: start at 0,0 and finishes at width,height: it means I select the whole graph area
      .on("end", updateChart) // Each time the brush selection changes, trigger the 'updateChart' function

  // Create the scatter variable: where both the circles and the brush take place
  var scatter = Svg.append('g')
    .attr("clip-path", "url(#clip)")

  // Add circles
  scatter
    .selectAll("circle")
    .data(data)
    .enter()
    .append("circle")
      .attr("cx", function (d) { return x(d.Date); } )
      .attr("cy", function (d) { return y(d.Depth); } )
      .attr("r", 8)
      .style("fill", function (d) { return color(d.Depth) } )
      .style("opacity", 0.5)

  // Add the brushing
  scatter
    .append("g")
      .attr("class", "brush")
      .call(brush);

  // A function that set idleTimeOut to null
  var idleTimeout
  function idled() { idleTimeout = null; }

  // A function that update the chart for given boundaries
  function updateChart() {
    let extent;
    extent = event.selection

    // If no selection, back to initial coordinate. Otherwise, update X axis domain
    if(!extent){
      if (!idleTimeout) return idleTimeout = setTimeout(idled, 350); // This allows to wait a little bit
      x.domain([ 0,40])
    }else{
      x.domain([ x.invert(extent[0]), x.invert(extent[1]) ])
      scatter.select(".brush").call(brush.move, null) // This remove the grey brush area as soon as the selection has been done
    }

    // Update axis and circle position
    xAxis.transition().duration(1000).call(axisBottom(x).tickFormat(xAxisTickFormat))
    scatter
      .selectAll("circle")
      .transition().duration(1000)
      .attr("cx", function (d) { return x(d.Date); } )
      .attr("cy", function (d) { return y(d.Depth); } )

    }



})